from django.views.generic import TemplateView, View, FormView
from django.shortcuts import resolve_url, redirect
from django.db.models import F, ExpressionWrapper, fields
from django.contrib import messages
from django.forms import ValidationError
from django.http import Http404

from datetime import datetime

import uuid

from .models import Playthrough
from .forms import PlaythroughForm

from dbsettings.models import Setting


class StartView(FormView):
    template_name = "start.html"
    form_class = PlaythroughForm

    def form_valid(self, form):
        session_id = self.request.session.get("session_id")

        if session_id is None:
            session_id = str(uuid.uuid4())
            self.request.session["session_id"] = session_id

        form.instance.session = session_id
        form.save()

        return redirect("redirect_out")

    def form_invalid(self, form):
        for field, errors in form.errors.items():
            for error in errors:
                messages.error(self.request, f"{field}: {error}")

        return super().form_invalid(form)

    def get(self, request, *args, **kwargs):
        session_id = request.session.get("session_id")

        if session_id is not None:
            try:
                playthrough = Playthrough.objects.get(session=session_id)
            except Playthrough.DoesNotExist:
                request.session["session_id"] = None
            else:
                if playthrough.end is None:
                    return redirect("redirect_out")
                else:
                    return redirect("end")

        return super().get(request, *args, **kwargs)


class EndView(TemplateView):
    template_name = "end.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        try:
            playthrough = Playthrough.objects.get(
                session=self.request.session.get("session_id")
            )
        except Playthrough.DoesNotExist:
            context["redirect"] = resolve_url("start")
            return context

        if playthrough.end is None:
            context["redirect"] = resolve_url("redirect_out")
            return context

        context["duration"] = playthrough.duration
        context["leaderboard"] = Playthrough.objects.filter(
            duration__isnull=False
        ).order_by("duration")[:10]
        return context


class RedirectOutView(TemplateView):
    template_name = "redirect_out.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["destination"] = Setting.objects.get(key="destination").value
        context["password"] = Setting.objects.get(key="password").value
        return context


class RedirectInView(View):
    template_name = "redirect_in.html"

    def get(self, request, *args, **kwargs):
        session_id = request.session.get("session_id")

        if session_id is None:
            return redirect("start")

        try:
            playthrough = Playthrough.objects.get(session=session_id)
        except Playthrough.DoesNotExist:
            return redirect("start")

        if playthrough.end is None:
            playthrough.end = datetime.now()
            playthrough.save()

        return redirect("end")


class RestartView(View):
    def get(self, request, *args, **kwargs):
        request.session["session_id"] = None
        return redirect("start")


class LeaderboardView(TemplateView):
    template_name = "leaderboard.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["leaderboard"] = Playthrough.objects.filter(
            duration__isnull=False
        ).order_by("duration")
        return context
